package dev.xpple.seedmapper.util.blocks;

import com.google.common.collect.ImmutableMap;
import kaptainwutax.biomeutils.biome.Biome;
import kaptainwutax.mcutils.state.Dimension;
import kaptainwutax.mcutils.version.MCVersion;
import net.minecraft.block.Block;

import java.util.Map;

import static kaptainwutax.biomeutils.biome.Biomes.*;
import static net.minecraft.block.Blocks.*;

public class SimpleBlockMap {
    // "ungenerate" different blocks to compare them with the seed
    private static final Map<MCVersion, Map<Biome, Map<Block, Integer>>> versionedBiomeMap = ImmutableMap.<MCVersion, Map<Biome, Map<Block, Integer>>>builder()
            .put(MCVersion.v1_16_5, ImmutableMap.<Biome, Map<Block, Integer>>builder()
                    .put(OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(GRAVEL, 1)
                            .put(SAND, 1)
                            .put(DIRT, 1)
                            .put(CLAY, 1)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .put(KELP, 9)
                            .put(KELP_PLANT, 9)
                            .build()
                    ).put(PLAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(TALL_GRASS, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(BEE_NEST, 0)
                            .put(AZURE_BLUET, 0)
                            .put(OXEYE_DAISY, 0)
                            .put(CORNFLOWER, 0)
                            .put(ORANGE_TULIP, 0)
                            .put(PINK_TULIP, 0)
                            .put(RED_TULIP, 0)
                            .put(WHITE_TULIP, 0)

                            .put(GRASS, 0)
                            .put(POPPY, 0)
                            .put(DANDELION, 0)
                            .build()
                    ).put(DESERT, ImmutableMap.<Block, Integer>builder()
                            .put(SAND, 1)
                            .put(SANDSTONE, 1)
                            .put(CACTUS, 0)
                            .put(DEAD_BUSH, 0)
                            .build()
                    ).put(MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .put(STONE, 1)
                            .put(INFESTED_STONE, 1)
                            .put(EMERALD_ORE, 1)
                            .put(COAL_ORE, 1)
                            .put(IRON_ORE, 1)
                            .put(SNOW, 0)
                            .put(ICE, 1)

                            .put(GRASS, 0)
                            .put(POPPY, 0)
                            .put(DANDELION, 0)
                            .build()
                    ).put(FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(BIRCH_LOG, 0)
                            .put(BIRCH_LEAVES, 0)
                            .put(BEE_NEST, 0)
                            .put(LILY_OF_THE_VALLEY, 0)
                            .put(LILAC, 0)
                            .put(ROSE_BUSH, 0)
                            .put(PEONY, 0)

                            .put(GRASS, 0)
                            .build()
                    ).put(TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(GRASS, 0)
                            .put(FERN, 0)
                            .put(LARGE_FERN, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .build()
                    ).put(SWAMP, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(WATER, 9)
                            .put(LILY_PAD, 0)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .put(CLAY, 1)
                            .put(VINE, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(BLUE_ORCHID, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .put(DEAD_BUSH, 0)
                            .build()
                    ).put(RIVER, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(GRASS_BLOCK, 1)
                            .put(DIRT, 1)
                            .put(CLAY, 1)
                            .put(SAND, 1)
                            .put(GRAVEL, 1)
                            .put(SUGAR_CANE, 0)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .build()
                    ).put(NETHER_WASTES, ImmutableMap.<Block, Integer>builder()
                            .put(NETHERRACK, 87)
                            .put(GLOWSTONE, 0)
                            .put(SOUL_SAND, 87)
                            .put(NETHER_QUARTZ_ORE, 87)
                            .put(NETHER_GOLD_ORE, 87)
                            .put(ANCIENT_DEBRIS, 87)
                            .put(MAGMA_BLOCK, 87)
                            .put(LAVA, 11)
                            .put(BLACKSTONE, 87)
                            .put(GRAVEL, 87)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .put(FIRE, 0)
                            .build()
                    ).put(THE_END, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, 121)
                            .put(OBSIDIAN, 0)
                            .put(BEDROCK, 0)
                            .build()
                    ).put(FROZEN_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(ICE, 1)
                            .put(SNOW_BLOCK, 1)
                            .put(PACKED_ICE, 1)
                            .put(BLUE_ICE, 1)
                            .put(GRAVEL, 1)
                            .put(SAND, 1)
                            .put(CLAY, 1)
                            .put(DIRT, 1)
                            .build()
                    ).put(FROZEN_RIVER, ImmutableMap.<Block, Integer>builder()
                            .put(ICE, 1)
                            .put(WATER, 9)
                            .put(GRASS_BLOCK, 1)
                            .put(DIRT, 1)
                            .put(CLAY, 1)
                            .put(SAND, 1)
                            .put(GRAVEL, 1)
                            .build()
                    ).put(SNOWY_TUNDRA, ImmutableMap.<Block, Integer>builder()
                            .put(SNOW, 0)
                            .put(GRASS_BLOCK, 1)
                            .put(ICE, 1)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .build()
                    ).put(SNOWY_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(SNOW, 0)
                            .put(GRASS_BLOCK, 1)
                            .put(ICE, 1)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .build()
                    ).put(MUSHROOM_FIELDS, ImmutableMap.<Block, Integer>builder()
                            .put(MYCELIUM, 1)
                            .put(RED_MUSHROOM_BLOCK, 0)
                            .put(BROWN_MUSHROOM_BLOCK, 0)
                            .put(MUSHROOM_STEM, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .build()
                    ).put(MUSHROOM_FIELD_SHORE, ImmutableMap.<Block, Integer>builder()
                            .put(MYCELIUM, 1)
                            .put(RED_MUSHROOM_BLOCK, 0)
                            .put(BROWN_MUSHROOM_BLOCK, 0)
                            .put(MUSHROOM_STEM, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .build()
                    ).put(BEACH, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(SAND, 1)
                            .put(SANDSTONE, 1)
                            .put(GRAVEL, 1)
                            .put(DIRT, 1)
                            .put(CLAY, 1)
                            .put(SUGAR_CANE, 0)
                            .build()
                    ).put(DESERT_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(SAND, 1)
                            .put(SANDSTONE, 1)
                            .put(CACTUS, 0)
                            .put(DEAD_BUSH, 0)
                            .build()
                    ).put(WOODED_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(BIRCH_LOG, 0)
                            .put(BIRCH_LEAVES, 0)
                            .put(BEE_NEST, 0)
                            .put(LILY_OF_THE_VALLEY, 0)
                            .put(LILAC, 0)
                            .put(ROSE_BUSH, 0)
                            .put(PEONY, 0)

                            .put(GRASS, 0)
                            .build()
                    ).put(TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(GRASS, 0)
                            .put(FERN, 0)
                            .put(LARGE_FERN, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .build()
                    ).put(MOUNTAIN_EDGE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .put(STONE, 1)
                            .put(INFESTED_STONE, 1)
                            .put(EMERALD_ORE, 1)
                            .put(COAL_ORE, 1)
                            .put(IRON_ORE, 1)
                            .put(SNOW, 0)
                            .put(ICE, 1)
                            .build()
                    ).put(JUNGLE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(VINE, 0)
                            .put(JUNGLE_LOG, 0)
                            .put(JUNGLE_LEAVES, 0)
                            .put(COCOA, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(MELON, 0)
                            .put(BAMBOO, 0)
                            .build()
                    ).put(JUNGLE_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(VINE, 0)
                            .put(JUNGLE_LOG, 0)
                            .put(JUNGLE_LEAVES, 0)
                            .put(COCOA, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(MELON, 0)
                            .put(BAMBOO, 0)
                            .build()
                    ).put(JUNGLE_EDGE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(VINE, 0)
                            .put(JUNGLE_LOG, 0)
                            .put(JUNGLE_LEAVES, 0)
                            .put(COCOA, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(MELON, 0)
                            .build()
                    ).put(DEEP_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(GRAVEL, 1)
                            .put(BUBBLE_COLUMN, 9)
                            .put(MAGMA_BLOCK, 1)
                            .put(OBSIDIAN, 1)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .put(KELP, 9)
                            .put(KELP_PLANT, 9)
                            .build()
                    ).put(STONE_SHORE, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(STONE, 1)
                            .put(GRAVEL, 1)
                            .build()
                    ).put(SNOWY_BEACH, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(SNOW, 0)
                            .put(ICE, 1)
                            .put(SAND, 1)
                            .put(SANDSTONE, 1)
                            .put(GRAVEL, 1)
                            .put(DIRT, 1)
                            .put(CLAY, 1)
                            .build()
                    ).put(BIRCH_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(BIRCH_LOG, 0)
                            .put(BIRCH_LEAVES, 0)
                            .put(BEE_NEST, 0)
                            .put(ROSE_BUSH, 0)
                            .put(LILAC, 0)
                            .put(PEONY, 0)
                            .put(LILY_OF_THE_VALLEY, 0)

                            .put(GRASS, 0)
                            .put(POPPY, 0)
                            .build()
                    ).put(BIRCH_FOREST_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(BIRCH_LOG, 0)
                            .put(BIRCH_LEAVES, 0)
                            .put(BEE_NEST, 0)
                            .put(ROSE_BUSH, 0)
                            .put(LILAC, 0)
                            .put(PEONY, 0)
                            .put(LILY_OF_THE_VALLEY, 0)

                            .put(GRASS, 0)
                            .put(DANDELION, 0)
                            .build()
                    ).put(DARK_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(DARK_OAK_LOG, 0)
                            .put(DARK_OAK_LEAVES, 0)
                            .put(BIRCH_LOG, 0)
                            .put(BIRCH_LEAVES, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(RED_MUSHROOM_BLOCK, 0)
                            .put(BROWN_MUSHROOM_BLOCK, 0)
                            .put(MUSHROOM_STEM, 0)
                            .put(ROSE_BUSH, 0)
                            .put(PEONY, 0)
                            .put(LILAC, 0)
                            .put(LILY_OF_THE_VALLEY, 0)

                            .put(GRASS, 0)
                            .put(POPPY, 0)
                            .build()
                    ).put(SNOWY_TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(SNOW, 0)
                            .put(GRASS, 0)
                            .put(FERN, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .put(LARGE_FERN, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .build()
                    ).put(SNOWY_TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(SNOW, 0)
                            .put(GRASS, 0)
                            .put(FERN, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .put(LARGE_FERN, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .build()
                    ).put(GIANT_TREE_TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(PODZOL, 1)
                            .put(COARSE_DIRT, 1)
                            .put(MOSSY_COBBLESTONE, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .put(FERN, 0)
                            .put(LARGE_FERN, 0)
                            .put(DEAD_BUSH, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .build()
                    ).put(GIANT_TREE_TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(PODZOL, 1)
                            .put(COARSE_DIRT, 1)
                            .put(MOSSY_COBBLESTONE, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .put(FERN, 0)
                            .put(LARGE_FERN, 0)
                            .put(DEAD_BUSH, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .build()
                    ).put(WOODED_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .put(STONE, 1)
                            .put(INFESTED_STONE, 1)
                            .put(EMERALD_ORE, 1)
                            .put(COAL_ORE, 1)
                            .put(IRON_ORE, 1)
                            .put(SNOW, 0)
                            .put(ICE, 1)

                            .put(GRASS, 0)
                            .put(POPPY, 0)
                            .build()
                    ).put(SAVANNA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(GRASS, 0)
                            .put(ACACIA_LOG, 0)
                            .put(ACACIA_LEAVES, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .build()
                    ).put(SAVANNA_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(TALL_GRASS, 0)
                            .put(ACACIA_LOG, 0)
                            .put(ACACIA_LEAVES, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .build()
                    ).put(BADLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, 1)
                            .put(TERRACOTTA, 1)
                            .put(WHITE_TERRACOTTA, 1)
                            .put(ORANGE_TERRACOTTA, 1)
                            .put(MAGENTA_TERRACOTTA, 1)
                            .put(LIGHT_BLUE_TERRACOTTA, 1)
                            .put(YELLOW_TERRACOTTA, 1)
                            .put(LIME_TERRACOTTA, 1)
                            .put(PINK_TERRACOTTA, 1)
                            .put(GRAY_TERRACOTTA, 1)
                            .put(LIGHT_GRAY_TERRACOTTA, 1)
                            .put(CYAN_TERRACOTTA, 1)
                            .put(PURPLE_TERRACOTTA, 1)
                            .put(BLUE_TERRACOTTA, 1)
                            .put(BROWN_TERRACOTTA, 1)
                            .put(GREEN_TERRACOTTA, 1)
                            .put(RED_TERRACOTTA, 1)
                            .put(BLACK_TERRACOTTA, 1)
                            .put(CACTUS, 0)
                            .put(DEAD_BUSH, 0)
                            .put(GOLD_ORE, 1)
                            .build()
                    ).put(WOODED_BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, 1)
                            .put(TERRACOTTA, 1)
                            .put(WHITE_TERRACOTTA, 1)
                            .put(ORANGE_TERRACOTTA, 1)
                            .put(MAGENTA_TERRACOTTA, 1)
                            .put(LIGHT_BLUE_TERRACOTTA, 1)
                            .put(YELLOW_TERRACOTTA, 1)
                            .put(LIME_TERRACOTTA, 1)
                            .put(PINK_TERRACOTTA, 1)
                            .put(GRAY_TERRACOTTA, 1)
                            .put(LIGHT_GRAY_TERRACOTTA, 1)
                            .put(CYAN_TERRACOTTA, 1)
                            .put(PURPLE_TERRACOTTA, 1)
                            .put(BLUE_TERRACOTTA, 1)
                            .put(BROWN_TERRACOTTA, 1)
                            .put(GREEN_TERRACOTTA, 1)
                            .put(RED_TERRACOTTA, 1)
                            .put(BLACK_TERRACOTTA, 1)
                            .put(CACTUS, 0)
                            .put(DEAD_BUSH, 0)
                            .put(GOLD_ORE, 1)
                            .put(GRASS_BLOCK, 1)
                            .put(COARSE_DIRT, 1)
                            .put(OAK_LOG, 1)
                            .put(OAK_LEAVES, 1)
                            .build()
                    ).put(BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, 1)
                            .put(TERRACOTTA, 1)
                            .put(WHITE_TERRACOTTA, 1)
                            .put(ORANGE_TERRACOTTA, 1)
                            .put(MAGENTA_TERRACOTTA, 1)
                            .put(LIGHT_BLUE_TERRACOTTA, 1)
                            .put(YELLOW_TERRACOTTA, 1)
                            .put(LIME_TERRACOTTA, 1)
                            .put(PINK_TERRACOTTA, 1)
                            .put(GRAY_TERRACOTTA, 1)
                            .put(LIGHT_GRAY_TERRACOTTA, 1)
                            .put(CYAN_TERRACOTTA, 1)
                            .put(PURPLE_TERRACOTTA, 1)
                            .put(BLUE_TERRACOTTA, 1)
                            .put(BROWN_TERRACOTTA, 1)
                            .put(GREEN_TERRACOTTA, 1)
                            .put(RED_TERRACOTTA, 1)
                            .put(BLACK_TERRACOTTA, 1)
                            .put(CACTUS, 0)
                            .put(DEAD_BUSH, 0)
                            .put(GOLD_ORE, 1)
                            .build()
                    ).put(SMALL_END_ISLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, 121)
                            .build()
                    ).put(END_MIDLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, 121)
                            .build()
                    ).put(END_HIGHLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, 121)
                            .put(CHORUS_PLANT, 0)
                            .put(CHORUS_FLOWER, 0)
                            .build()
                    ).put(END_BARRENS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, 121)
                            .build()
                    ).put(WARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(GRAVEL, 1)
                            .put(SAND, 1)
                            .put(DIRT, 1)
                            .put(CLAY, 1)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .put(KELP, 9)
                            .put(KELP_PLANT, 9)
                            .put(BUBBLE_COLUMN, 9)
                            .put(MAGMA_BLOCK, 1)
                            .put(OBSIDIAN, 1)
                            .put(BRAIN_CORAL_BLOCK, 1)
                            .put(BRAIN_CORAL_FAN, 1)
                            .put(BUBBLE_CORAL_BLOCK, 1)
                            .put(BUBBLE_CORAL_FAN, 1)
                            .put(DEAD_BRAIN_CORAL_BLOCK, 1)
                            .put(DEAD_BRAIN_CORAL_FAN, 1)
                            .put(DEAD_BUBBLE_CORAL_BLOCK, 1)
                            .put(DEAD_BUBBLE_CORAL_FAN, 1)
                            .put(FIRE_CORAL_BLOCK, 1)
                            .put(FIRE_CORAL_FAN, 1)
                            .put(DEAD_FIRE_CORAL_BLOCK, 1)
                            .put(DEAD_FIRE_CORAL_FAN, 1)
                            .put(HORN_CORAL_BLOCK, 1)
                            .put(HORN_CORAL_FAN, 1)
                            .put(TUBE_CORAL_BLOCK, 1)
                            .put(TUBE_CORAL_FAN, 1)
                            .put(BRAIN_CORAL_WALL_FAN, 1)
                            .put(BRAIN_CORAL, 1)
                            .put(BUBBLE_CORAL_WALL_FAN, 1)
                            .put(BUBBLE_CORAL, 1)
                            .put(DEAD_BRAIN_CORAL_WALL_FAN, 1)
                            .put(DEAD_BRAIN_CORAL, 1)
                            .put(DEAD_BUBBLE_CORAL_WALL_FAN, 1)
                            .put(DEAD_BUBBLE_CORAL, 1)
                            .put(FIRE_CORAL_WALL_FAN, 1)
                            .put(FIRE_CORAL, 1)
                            .put(DEAD_FIRE_CORAL_WALL_FAN, 1)
                            .put(DEAD_FIRE_CORAL, 1)
                            .put(HORN_CORAL_WALL_FAN, 1)
                            .put(HORN_CORAL, 1)
                            .put(TUBE_CORAL_WALL_FAN, 1)
                            .put(TUBE_CORAL, 1)
                            .put(SEA_PICKLE, 1)
                            .build()
                    ).put(LUKEWARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(GRAVEL, 1)
                            .put(SAND, 1)
                            .put(DIRT, 1)
                            .put(CLAY, 1)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .put(KELP, 9)
                            .put(KELP_PLANT, 9)
                            .build()
                    ).put(COLD_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(GRAVEL, 1)
                            .put(SAND, 1)
                            .put(DIRT, 1)
                            .put(CLAY, 1)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .put(KELP, 9)
                            .put(KELP_PLANT, 9)
                            .build()
                    ).put(DEEP_WARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(SAND, 1)
                            .put(BUBBLE_COLUMN, 9)
                            .put(MAGMA_BLOCK, 1)
                            .put(OBSIDIAN, 1)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .build()
                    ).put(DEEP_LUKEWARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(SAND, 1)
                            .put(BUBBLE_COLUMN, 9)
                            .put(MAGMA_BLOCK, 1)
                            .put(OBSIDIAN, 1)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .put(KELP, 9)
                            .put(KELP_PLANT, 9)
                            .build()
                    ).put(DEEP_COLD_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(GRAVEL, 1)
                            .put(BUBBLE_COLUMN, 9)
                            .put(MAGMA_BLOCK, 1)
                            .put(OBSIDIAN, 1)
                            .put(SEAGRASS, 9)
                            .put(TALL_SEAGRASS, 9)
                            .put(KELP, 9)
                            .put(KELP_PLANT, 9)
                            .build()
                    ).put(DEEP_FROZEN_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, 9)
                            .put(SNOW_BLOCK, 1)
                            .put(PACKED_ICE, 1)
                            .put(ICE, 1)
                            .put(BLUE_ICE, 1)
                            .put(GRAVEL, 1)
                            .put(BUBBLE_COLUMN, 9)
                            .put(MAGMA_BLOCK, 1)
                            .put(OBSIDIAN, 1)
                            .build()
                    ).put(THE_VOID, ImmutableMap.<Block, Integer>builder()
                            .put(COBBLESTONE, 1)
                            .put(STONE, 1)
                            .build()
                    ).put(SUNFLOWER_PLAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(TALL_GRASS, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(BEE_NEST, 0)
                            .put(AZURE_BLUET, 0)
                            .put(OXEYE_DAISY, 0)
                            .put(CORNFLOWER, 0)
                            .put(ORANGE_TULIP, 0)
                            .put(PINK_TULIP, 0)
                            .put(RED_TULIP, 0)
                            .put(WHITE_TULIP, 0)
                            .put(SUNFLOWER, 0)
                            .build()
                    ).put(DESERT_LAKES, ImmutableMap.<Block, Integer>builder()
                            .put(SAND, 1)
                            .put(SANDSTONE, 1)
                            .put(CACTUS, 0)
                            .put(DEAD_BUSH, 0)
                            .put(WATER, 9)
                            .put(SUGAR_CANE, 0)
                            .build()
                    ).put(GRAVELLY_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRAVEL, 1)
                            .put(GRASS_BLOCK, 1)
                            .put(SNOW, 0)
                            .put(STONE, 1)
                            .put(INFESTED_STONE, 1)
                            .put(EMERALD_ORE, 1)
                            .put(COAL_ORE, 1)
                            .put(IRON_ORE, 1)
                            .build()
                    ).put(FLOWER_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(BIRCH_LOG, 0)
                            .put(BIRCH_LEAVES, 0)
                            .put(BEE_NEST, 0)
                            .put(LILY_OF_THE_VALLEY, 0)
                            .put(LILAC, 0)
                            .put(ROSE_BUSH, 0)
                            .put(PEONY, 0)
                            .put(ALLIUM, 0)
                            .build()
                    ).put(TAIGA_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(GRASS, 0)
                            .put(FERN, 0)
                            .put(LARGE_FERN, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .build()
                    ).put(SWAMP_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(WATER, 9)
                            .put(LILY_PAD, 0)
                            .put(CLAY, 1)
                            .put(VINE, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(BLUE_ORCHID, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .build()
                    ).put(ICE_SPIKES, ImmutableMap.<Block, Integer>builder()
                            .put(PACKED_ICE, 0)
                            .put(SNOW_BLOCK, 1)
                            .put(SNOW, 0)
                            .put(DIRT, 1)
                            .put(ICE, 1)
                            .build()
                    ).put(MODIFIED_JUNGLE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(VINE, 0)
                            .put(JUNGLE_LOG, 0)
                            .put(JUNGLE_LEAVES, 0)
                            .put(COCOA, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(MELON, 0)
                            .put(BAMBOO, 0)
                            .build()
                    ).put(MODIFIED_JUNGLE_EDGE,  ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(VINE, 0)
                            .put(JUNGLE_LOG, 0)
                            .put(JUNGLE_LEAVES, 0)
                            .put(COCOA, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(MELON, 0)
                            .build()
                    ).put(TALL_BIRCH_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(BIRCH_LOG, 0)
                            .put(BIRCH_LEAVES, 0)
                            .put(BEE_NEST, 0)
                            .put(ROSE_BUSH, 0)
                            .put(LILAC, 0)
                            .put(PEONY, 0)
                            .put(LILY_OF_THE_VALLEY, 0)
                            .build()
                    ).put(TALL_BIRCH_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(BIRCH_LOG, 0)
                            .put(BIRCH_LEAVES, 0)
                            .put(BEE_NEST, 0)
                            .put(ROSE_BUSH, 0)
                            .put(LILAC, 0)
                            .put(PEONY, 0)
                            .put(LILY_OF_THE_VALLEY, 0)
                            .build()
                    ).put(DARK_FOREST_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(DARK_OAK_LOG, 0)
                            .put(DARK_OAK_LEAVES, 0)
                            .put(BIRCH_LOG, 0)
                            .put(BIRCH_LEAVES, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(RED_MUSHROOM_BLOCK, 0)
                            .put(BROWN_MUSHROOM_BLOCK, 0)
                            .put(MUSHROOM_STEM, 0)
                            .put(ROSE_BUSH, 0)
                            .put(PEONY, 0)
                            .put(LILAC, 0)
                            .put(LILY_OF_THE_VALLEY, 0)
                            .build()
                    ).put(SNOWY_TAIGA_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(SNOW, 0)
                            .put(GRASS, 0)
                            .put(FERN, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .put(LARGE_FERN, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .build()
                    ).put(GIANT_SPRUCE_TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(PODZOL, 1)
                            .put(COARSE_DIRT, 1)
                            .put(MOSSY_COBBLESTONE, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .put(FERN, 0)
                            .put(LARGE_FERN, 0)
                            .put(DEAD_BUSH, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .build()
                    ).put(GIANT_SPRUCE_TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(PODZOL, 1)
                            .put(COARSE_DIRT, 1)
                            .put(MOSSY_COBBLESTONE, 0)
                            .put(SPRUCE_LOG, 0)
                            .put(SPRUCE_LEAVES, 0)
                            .put(FERN, 0)
                            .put(LARGE_FERN, 0)
                            .put(DEAD_BUSH, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .put(SWEET_BERRY_BUSH, 0)
                            .build()
                    ).put(MODIFIED_GRAVELLY_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRAVEL, 1)
                            .put(GRASS_BLOCK, 1)
                            .put(SNOW, 0)
                            .put(STONE, 1)
                            .put(INFESTED_STONE, 1)
                            .put(EMERALD_ORE, 1)
                            .put(COAL_ORE, 1)
                            .put(IRON_ORE, 1)
                            .build()
                    ).put(SHATTERED_SAVANNA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(COARSE_DIRT, 1)
                            .put(STONE, 1)
                            .put(TALL_GRASS, 0)
                            .put(ACACIA_LOG, 0)
                            .put(ACACIA_LEAVES, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .build()
                    ).put(SHATTERED_SAVANNA_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(COARSE_DIRT, 1)
                            .put(STONE, 1)
                            .put(TALL_GRASS, 0)
                            .put(ACACIA_LOG, 0)
                            .put(ACACIA_LEAVES, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .build()
                    ).put(ERODED_BADLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, 1)
                            .put(TERRACOTTA, 1)
                            .put(WHITE_TERRACOTTA, 1)
                            .put(ORANGE_TERRACOTTA, 1)
                            .put(MAGENTA_TERRACOTTA, 1)
                            .put(LIGHT_BLUE_TERRACOTTA, 1)
                            .put(YELLOW_TERRACOTTA, 1)
                            .put(LIME_TERRACOTTA, 1)
                            .put(PINK_TERRACOTTA, 1)
                            .put(GRAY_TERRACOTTA, 1)
                            .put(LIGHT_GRAY_TERRACOTTA, 1)
                            .put(CYAN_TERRACOTTA, 1)
                            .put(PURPLE_TERRACOTTA, 1)
                            .put(BLUE_TERRACOTTA, 1)
                            .put(BROWN_TERRACOTTA, 1)
                            .put(GREEN_TERRACOTTA, 1)
                            .put(RED_TERRACOTTA, 1)
                            .put(BLACK_TERRACOTTA, 1)
                            .put(CACTUS, 0)
                            .put(DEAD_BUSH, 0)
                            .put(GOLD_ORE, 1)
                            .build()
                    ).put(MODIFIED_WOODED_BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, 1)
                            .put(TERRACOTTA, 1)
                            .put(WHITE_TERRACOTTA, 1)
                            .put(ORANGE_TERRACOTTA, 1)
                            .put(MAGENTA_TERRACOTTA, 1)
                            .put(LIGHT_BLUE_TERRACOTTA, 1)
                            .put(YELLOW_TERRACOTTA, 1)
                            .put(LIME_TERRACOTTA, 1)
                            .put(PINK_TERRACOTTA, 1)
                            .put(GRAY_TERRACOTTA, 1)
                            .put(LIGHT_GRAY_TERRACOTTA, 1)
                            .put(CYAN_TERRACOTTA, 1)
                            .put(PURPLE_TERRACOTTA, 1)
                            .put(BLUE_TERRACOTTA, 1)
                            .put(BROWN_TERRACOTTA, 1)
                            .put(GREEN_TERRACOTTA, 1)
                            .put(RED_TERRACOTTA, 1)
                            .put(BLACK_TERRACOTTA, 1)
                            .put(CACTUS, 0)
                            .put(DEAD_BUSH, 0)
                            .put(GOLD_ORE, 1)
                            .put(GRASS_BLOCK, 1)
                            .put(COARSE_DIRT, 1)
                            .put(OAK_LOG, 1)
                            .put(OAK_LEAVES, 1)
                            .build()
                    ).put(MODIFIED_BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, 1)
                            .put(TERRACOTTA, 1)
                            .put(WHITE_TERRACOTTA, 1)
                            .put(ORANGE_TERRACOTTA, 1)
                            .put(MAGENTA_TERRACOTTA, 1)
                            .put(LIGHT_BLUE_TERRACOTTA, 1)
                            .put(YELLOW_TERRACOTTA, 1)
                            .put(LIME_TERRACOTTA, 1)
                            .put(PINK_TERRACOTTA, 1)
                            .put(GRAY_TERRACOTTA, 1)
                            .put(LIGHT_GRAY_TERRACOTTA, 1)
                            .put(CYAN_TERRACOTTA, 1)
                            .put(PURPLE_TERRACOTTA, 1)
                            .put(BLUE_TERRACOTTA, 1)
                            .put(BROWN_TERRACOTTA, 1)
                            .put(GREEN_TERRACOTTA, 1)
                            .put(RED_TERRACOTTA, 1)
                            .put(BLACK_TERRACOTTA, 1)
                            .put(CACTUS, 0)
                            .put(DEAD_BUSH, 0)
                            .put(GOLD_ORE, 1)
                            .build()
                    ).put(BAMBOO_JUNGLE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(PODZOL, 1)
                            .put(BAMBOO, 0)
                            .put(VINE, 0)
                            .put(JUNGLE_LOG, 0)
                            .put(JUNGLE_LEAVES, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(MELON, 0)
                            .build()
                    ).put(BAMBOO_JUNGLE_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, 1)
                            .put(PODZOL, 1)
                            .put(BAMBOO, 0)
                            .put(VINE, 0)
                            .put(JUNGLE_LOG, 0)
                            .put(JUNGLE_LEAVES, 0)
                            .put(OAK_LOG, 0)
                            .put(OAK_LEAVES, 0)
                            .put(MELON, 0)
                            .build()
                    ).put(SOUL_SAND_VALLEY, ImmutableMap.<Block, Integer>builder()
                            .put(NETHERRACK, 87)
                            .put(LAVA, 11)
                            .put(GRAVEL, 87)
                            .put(SOUL_SAND, 87)
                            .put(SOUL_SOIL, 87)
                            .put(BASALT, 87)
                            .put(BLACKSTONE, 87)
                            .put(BONE_BLOCK, 87)
                            .put(GLOWSTONE, 0)
                            .put(CRIMSON_ROOTS, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .put(SOUL_FIRE, 0)
                            .build()
                    ).put(CRIMSON_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(CRIMSON_NYLIUM, 87)
                            .put(NETHERRACK, 87)
                            .put(CRIMSON_STEM, 0)
                            .put(NETHER_WART_BLOCK, 87)
                            .put(BLACKSTONE, 87)
                            .put(GRAVEL, 87)
                            .put(SHROOMLIGHT, 0)
                            .put(CRIMSON_ROOTS, 0)
                            .put(CRIMSON_FUNGUS, 0)
                            .put(WARPED_FUNGUS, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .put(WEEPING_VINES, 0)
                            .put(WEEPING_VINES_PLANT, 0)
                            .build()
                    ).put(WARPED_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(WARPED_NYLIUM, 87)
                            .put(NETHERRACK, 87)
                            .put(WARPED_STEM, 0)
                            .put(WARPED_WART_BLOCK, 87)
                            .put(BLACKSTONE, 87)
                            .put(GRAVEL, 87)
                            .put(SHROOMLIGHT, 0)
                            .put(WARPED_ROOTS, 0)
                            .put(WARPED_FUNGUS, 0)
                            .put(CRIMSON_FUNGUS, 0)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .put(NETHER_SPROUTS, 0)
                            .put(TWISTING_VINES, 0)
                            .build()
                    ).put(BASALT_DELTAS, ImmutableMap.<Block, Integer>builder()
                            .put(NETHERRACK, 87)
                            .put(BASALT, 87)
                            .put(BLACKSTONE, 87)
                            .put(MAGMA_BLOCK, 87)
                            .put(LAVA, 11)
                            .put(GRAVEL, 87)
                            .put(RED_MUSHROOM, 0)
                            .put(BROWN_MUSHROOM, 0)
                            .build()
                    )
                    .build()
            )
            .build();

    private static final Map<Block, Integer> overworld = ImmutableMap.<Block, Integer>builder()
            .put(AIR, 0)
            .put(CAVE_AIR, 1)
            .put(VOID_AIR, 0)
            //.put(BEDROCK, 7)
            .put(BEDROCK, 1)
            .put(STONE, 1)
            .put(DIRT, 1)
            .put(WATER, 1)
            .put(LAVA, 1)
            .put(COAL_ORE, 1)
            .put(IRON_ORE, 1)
            .put(GOLD_ORE, 1)
            .put(DIAMOND_ORE, 1)
            .put(EMERALD_ORE, 1)
            .put(LAPIS_ORE, 1)
            .put(REDSTONE_ORE, 1)
            .put(ANDESITE, 1)
            .put(DIORITE, 1)
            .put(GRANITE, 1)
            .put(GRAVEL, 1)
            .build();

    private static final Map<Block, Integer> nether = ImmutableMap.<Block, Integer>builder()
            .put(AIR, 0)
            .put(CAVE_AIR, 87)
            .put(VOID_AIR, 0)
            //.put(BEDROCK, 7)
            .put(BEDROCK, 87)
            .put(NETHERRACK, 87)
            .put(LAVA, 11)
            .put(NETHER_QUARTZ_ORE, 87)
            .put(NETHER_GOLD_ORE, 87)
            .put(BLACKSTONE, 87)
            .put(BASALT, 87)
            .put(ANCIENT_DEBRIS, 87)
            .build();

    private static final Map<Block, Integer> end = ImmutableMap.<Block, Integer>builder()
            .put(AIR, 0)
            .put(CAVE_AIR, 121)
            .put(VOID_AIR, 0)
            .put(END_STONE, 121)
            .build();

    private Map<Block, Integer> map;
    private final Dimension dimension;

    public SimpleBlockMap(Dimension dimension) {
        this(dimension, PLAINS);
    }

    public SimpleBlockMap(Dimension dimension, Biome biome) {
        this(MCVersion.v1_16_5, dimension, biome);
    }

    public SimpleBlockMap(MCVersion mcVersion, Dimension dimension, Biome biome) {
        this.map = versionedBiomeMap.get(mcVersion).get(biome);
        this.dimension = dimension;
    }

    public void setBiome(Biome biome) {
        this.map = versionedBiomeMap.get(MCVersion.v1_16_5).get(biome);
    }

    public int get(Block block) {
        Integer biomeBlock = this.map.get(block);
        if (biomeBlock == null) {
            if (this.dimension == Dimension.OVERWORLD) {
                return overworld.getOrDefault(block, -1);
            } else if (this.dimension == Dimension.NETHER) {
                return nether.getOrDefault(block, -1);
            } else {
                return end.getOrDefault(block, -1);
            }
        }
        return biomeBlock;
    }
}
